# Utilizing the MCP23017 in multitasking mode by using asyncio
# 
# Here we use an MCP23017 peripheral extender (this is the I2C version)
# GPB0 controls a LED, and the led blinking is controlled by two pushbuttons 
# (UP connected to GPA7, DOWN connected to GPA4)
# There is a third pusbutton as well, connected to GPB1, which switches 
# the builtin LED (board.LED) on and off.
#
# The program is based on the following tutorials:
# https://learn.adafruit.com/cooperative-multitasking-in-circuitpython-with-asyncio/
# https://learn.adafruit.com/using-mcp23008-mcp23017-with-circuitpython
#
# Tested with:
# Adafruit CircuitPython 7.3.3 on 2022-08-29; stm32f411ce-blackpill-with-flash with STM32F411CE


import asyncio
import board
import busio
import digitalio
from adafruit_debouncer import Debouncer
from adafruit_mcp230xx.mcp23017 import MCP23017

class Interval:
    """Simple class to hold an interval value. Use .value to to read or write."""

    def __init__(self, initial_interval):
        self.value = initial_interval
        
interval = Interval(0.25)
led = digitalio.DigitalInOut(board.LED)
led.direction = digitalio.Direction.OUTPUT
led.value = True

# Initialize the I2C bus:
i2c = busio.I2C(board.SCL, board.SDA)
mcp = MCP23017(i2c, address=0x20)  # MCP23017 with A0,A1,A2 grounded

pin8 = mcp.get_pin(8)   # GPB0
pin9 = mcp.get_pin(9)   # GPB1 
pin7 = mcp.get_pin(7)   # GPA7
pin4 = mcp.get_pin(4)   # GPA4

# Setup pushbutton pins as input with internal pull-up enabled
pin9.direction = digitalio.Direction.INPUT
pin9.pull = digitalio.Pull.UP
pin7.direction = digitalio.Direction.INPUT
pin7.pull = digitalio.Pull.UP
pin4.direction = digitalio.Direction.INPUT
pin4.pull = digitalio.Pull.UP

key = Debouncer(pin9)
btn_up = Debouncer(pin7)
btn_dn = Debouncer(pin4)

# Setup pin8 as an output that's at a high logic level.
pin8.switch_to_output(value=True)

async def interval_buttons():
    # Monitor two buttons: one lengthens the interval, the other shortens it.
    while True:
        btn_up.update()
        if btn_up.fell:
            interval.value += 0.1
        btn_dn.update()
        if btn_dn.fell:
            interval.value = max(0.1, interval.value - 0.1)
        await asyncio.sleep(0.02)

async def led_switch():
    while True:
        key.update()
        if key.fell:
            led.value = not led.value  # Toggle builtin LED's state 
        await asyncio.sleep(0.02)
        
async def led_blink():
    while True:
        pin8.value = True
        await asyncio.sleep(interval.value)  # Don't forget the "await"!
        pin8.value = False
        await asyncio.sleep(interval.value)  # Don't forget the "await"!        

async def main():
    task1 = asyncio.create_task(interval_buttons())
    task2 = asyncio.create_task(led_switch())
    task3 = asyncio.create_task(led_blink())
    await asyncio.gather(task1, task2, task3)  # Don't forget "await"!

asyncio.run(main())
